from Ui_NumberRecognizer import Ui_NumberRecognizer
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from opencv import *
from ctypes import *
import os


class NumberRecognizer(QDialog, Ui_NumberRecognizer):
    def __init__(self):
        QDialog.__init__(self)
        self.setupUi(self)
        self.frame = None
        self.capture = None
        self.extDLL = None
        self.baseNumberImageList = []
        self.binaryBaseNumberImageList = []

    def loadBaseImage(self):
        for index in range(10):
            if os.path.exists("./Images/"+str(index)+".bmp"):
                self.baseNumberImageList.append(cvLoadImage("./Images/"+str(index)+".bmp", 0))
            else:
                return False
        return True

    def initCamera(self):
        self.capture = cvCaptureFromCAM(0);
        return self.capture
        
    def timerEvent(self, ev):
        if(cvGrabFrame(self.capture) == None):
            return
        initFrame = cvRetrieveFrame(self.capture)
        if(initFrame == None):
            returen
        if(self.frame == None):
            self.frame = cvCreateImage(cvSize(initFrame.width, initFrame.height), IPL_DEPTH_8U, initFrame.nChannels)
        if(initFrame.origin == IPL_ORIGIN_TL):
            cvCopy(initFrame, self.frame, 0)
        else:
            cvFlip(initFrame, self.frame, 0)
        self.processFrame(self.frame)

    def processFrame(self, frame):
        inputImage = cvCreateImage(cvGetSize(self.frame), IPL_DEPTH_8U, self.frame.nChannels)
        cvCopy(self.frame, inputImage);
        inputImageGray = cvCreateImage(cvGetSize(self.frame), IPL_DEPTH_8U, 1)
        cvCvtColor(inputImage, inputImageGray, CV_BGR2GRAY)
        inputImageThreshold = cvCreateImage(cvGetSize(self.frame), IPL_DEPTH_8U, 1)
        cvThreshold(inputImageGray, inputImageThreshold, 50, 255, CV_THRESH_BINARY);

        searchNumberArea = self.extDLL.searchNumberArea
        cropNumberArea = self.extDLL.cropNumberArea
        compareImage = self.extDLL.compareImage
        
        bottom = c_int(0)
        top = c_int(10000)
        right = c_int(0)
        left = c_int(10000)
        searchNumberArea(inputImageThreshold.imageData, inputImageThreshold.width, inputImageThreshold.height, addressof(bottom), addressof(top), addressof(right), addressof(left))
        croppedInputImage = cvCreateImage(cvSize(abs(right.value-left.value), abs(bottom.value-top.value)), IPL_DEPTH_8U, 1)
        cropNumberArea(inputImageThreshold.imageData, croppedInputImage.imageData, inputImageThreshold.widthStep, croppedInputImage.widthStep, bottom, top, right, left)
        rect = cvRect(left.value, top.value, abs(right.value-left.value), abs(bottom.value-top.value))
        cvRectangle(frame, cvPoint(rect.x, rect.y), cvPoint(rect.x+rect.width, rect.y+rect.height), CV_RGB(255, 0, 0), 2)
        resizedCroppedNumberImage = cvCreateImage(cvGetSize(self.baseNumberImageList[0]), IPL_DEPTH_8U, 1)
        cvResize(croppedInputImage, resizedCroppedNumberImage, CV_INTER_LINEAR)
        
        countList = []
        for index in range(10):
            countValue = compareImage(self.baseNumberImageList[index].imageData, resizedCroppedNumberImage.imageData, self.baseNumberImageList[index].width, self.baseNumberImageList[index].height)
            countList.append(countValue)
        
        cvSaveImage("savedImage.png", frame)
        viewImage = QPixmap('savedImage.png')
        if viewImage.width != 320 and viewImage.height != 240:
            viewImage = viewImage.scaledToWidth(320).scaledToHeight(240)
        self.ImageLabel.setPixmap(viewImage)
        self.resultLabel.setText(QString(str(countList.index(max(countList)))))
        
        del inputImage
        del inputImageGray
        del inputImageThreshold
        del croppedInputImage
        del resizedCroppedNumberImage
        os.remove("savedImage.png")


if __name__ == "__main__":
    app = QApplication(sys.argv)
    numberRecognizer = NumberRecognizer()
    if numberRecognizer.initCamera() == None:
        QMessageBox.warning(numberRecognizer, "Information", "Please connect webcam")
        sys.exit()
    if numberRecognizer.loadBaseImage() == False:
        QMessageBox.warning(numberRecognizer, "Information", "Base Number Images are not found")
        del numberRecognizer.capture
        sys.exit()
    if os.path.exists("./ExtDLL.dll") == False:
        QMessageBox.warning(numberRecognizer, "Information", "External DLL File is not found")
        del numberRecognizer.capture
        sys.exit()
    numberRecognizer.extDLL = cdll.LoadLibrary("./ExtDLL.dll")
    numberRecognizer.startTimer(100)
    numberRecognizer.show()
    sys.exit(app.exec_())